#include <pololu/3pi.h>
#include <stdio.h>
#include <math.h>

void programCountdown()
{

	red_led(0);
	green_led(0);

	clear();
	print_long(read_battery_millivolts_3pi());
	print("mV");
	lcd_goto_xy(0, 1);
	print("Press B");

	while(!button_is_pressed(BUTTON_B));

	clear();
	print("Starting");

	lcd_goto_xy(0, 1);
	print("in 3...");
	play_note(C(5), 100, 15);
	delay_ms(1000);

	lcd_goto_xy(0, 1);
	print("in 2...");
	play_note(C(5), 100, 15);
	delay_ms(1000);

	lcd_goto_xy(0, 1);
	print("in 1...");
	play_note(C(5), 100, 15);
	delay_ms(1000);

	play_note(B(5), 300, 15);

	clear();
	print("Any btn");
	lcd_goto_xy(0, 1);
	print("to halt");

}

void steer(int angle, int distance)
{

	const int MOTOR_SPEED = 40;
	const float ANGLE_WINDOW = 30.0;
	const float WINDOW_SLOW_THRESHOLD = 30.0;

	float speedScale;
	float turnStrength = 0.7;
	float distFromWindow;

	int left;
	int right;

	if(distance > 0)
	{
		
		if(angle > ANGLE_WINDOW)
		{

			distFromWindow = angle - ANGLE_WINDOW;
			if(distFromWindow < WINDOW_SLOW_THRESHOLD)
			{
				turnStrength = 0.5;
			}

			set_motors(MOTOR_SPEED * turnStrength, -MOTOR_SPEED * turnStrength);

		}
		else if(angle < -ANGLE_WINDOW)
		{

			distFromWindow = -angle + ANGLE_WINDOW;
			if(distFromWindow < WINDOW_SLOW_THRESHOLD)
			{
				turnStrength = 0.5;
			}

			set_motors(-MOTOR_SPEED * turnStrength, MOTOR_SPEED * turnStrength);
		}
		else
		{
			
			turnStrength = fabs((float)angle / ANGLE_WINDOW) * 0.5;

			if(distance == 1)
			{
				speedScale = 0.5;
			}
			else
			{
				speedScale = 1.0;
			}

			if(angle > 0)
			{
				left = MOTOR_SPEED * speedScale;
				right = MOTOR_SPEED * (1.0 - turnStrength) * speedScale;
			}
			else
			{
				left = MOTOR_SPEED * (1.0 - turnStrength) * speedScale;
				right = MOTOR_SPEED * speedScale;
			}

			set_motors(left, right);
			
		}
		
	}
	else
	{
		set_motors(0, 0);
	}

}

int main(void)
{

	const char POWER_ON[] = {"!O5MST1200aaabbb"};
	const int MAX_DATA_TRIES = 15;

	char buffer;
	int angle = 0;
	int distance = 0;

	uint8_t active = 0;
	int dataTries = 0;

	serial_set_baud_rate(9600);
	play(POWER_ON);
	
	programCountdown();

	while(1)
	{

		// ensure that we only act on data we've received recently
		if(serial_receive_blocking(&buffer, 1, 25) == 0)
		{

			angle = ((buffer & 63) * 6) - 180;						// 6 bits for angle is ~6 degree precision
			distance = (int)((buffer & 192) >> 6);					// 2 bits for distance

			active = 1;
			dataTries = 0;

		}
		else		// we haven't received any data recently
		{

			dataTries ++;
			if(dataTries > MAX_DATA_TRIES)		// delay in comms will pause the robot
			{
				set_motors(0, 0);
				active = 0;
			}

		}

		if(active)
		{
			steer(angle, distance);
		}

		clear();
		print("d: ");
		print_long(distance);
		lcd_goto_xy(0, 1);
		print("a: ");
		print_long(angle);

	}

}
