#include "formation.h"
#include "pawprint.h"
#include "util.h"

#include <vector>
#include <opencv/cv.h>

using namespace std;

const int Formation::HORIZONTAL_LINE = 0;
const int Formation::DIAGONAL_LINE = 1;
const int Formation::VERTICAL_LINE = 2;
const int Formation::CIRCLE = 3;

int Formation::currentFormation = CIRCLE;

int Formation::nextFormation()
{
	
	if(++currentFormation > 3)
	{
		currentFormation = 0;
	}

	return currentFormation;
	
}

void Formation::getFormationPoints(int robotCount, vector<Point2f> &points, Size frameSize)
{

	const float FORMATION_START = 0.0;
	const float FORMATION_END = 1.0;
	
	Point2f start;
	Point2f end;
	Point2f point;
	Point2f relative;
	
	int pointDistance;
	int i;
	
	points.clear();
	
	if(robotCount == 1)
	{
		// regardless of the formation type, a single robot will always move to the center of the arena
		points.push_back(Point2f(frameSize.width * 0.5, frameSize.height * 0.5));
	}
	else if(robotCount > 0)
	{

		if(currentFormation == VERTICAL_LINE)
		{
		
			start = Point2f(frameSize.width * 0.5, frameSize.height * FORMATION_START);
			end = Point2f(frameSize.width * 0.5, frameSize.height * FORMATION_END);
		
			pointDistance = Util::length(end - start) / (robotCount + 1);
			point = start + Point2f(0.0, pointDistance);
		
			for(i = 0; i < robotCount; i ++)
			{
				points.push_back(point);
				point = point + Point2f(0.0, pointDistance);
			}
		
		}
		else if(currentFormation == DIAGONAL_LINE)
		{
		
			start = Point2f(frameSize.width * FORMATION_START, frameSize.height * FORMATION_START);
			end = Point2f(frameSize.width * FORMATION_END, frameSize.height * FORMATION_END);
			
			relative = Util::normalize(end - start);
			pointDistance = (end.x - start.x) / (robotCount);
			point = start + (relative * pointDistance);
			
			for(i = 0; i < robotCount; i ++)
			{
				points.push_back(point);
				point = point + relative * pointDistance;
			}
		
		}
		else if(currentFormation == HORIZONTAL_LINE)
		{
		
			start = Point2f(frameSize.width * FORMATION_START, frameSize.height * 0.5);
			end = Point2f(frameSize.width * FORMATION_END, frameSize.height * 0.5);
		
			pointDistance = (end.x - start.x) / (robotCount + 1);
			point = start + Point2f(pointDistance, 0.0);
		
			for(i = 0; i < robotCount; i ++)
			{
				points.push_back(point);
				point = point + Point2f(pointDistance, 0.0);
			}
		
		}
		else if(currentFormation == CIRCLE)
		{
		
			start = Point2f(frameSize.width * 0.5, frameSize.height * 0.5);
			pointDistance = 0;
			
			for(i = 0; i < robotCount; i ++)
			{
				
				point = start + Util::vecAtAngle(pointDistance) * frameSize.width * 0.3;
				pointDistance += (360 / robotCount);
				
				points.push_back(point);
				
			}
		
		}
	
	}

}

int Formation::getFurthestPrintFromPoint(Point2f point, vector<PawPrint*> prints)
{

	int i;
	int result = 0;
	int furthest = 0;
	int dist;
	
	for(i = 0; i < (int)prints.size(); i ++)
	{
	
		dist = Util::length(prints[i] -> getCenter() - point);
		if(dist > furthest)
		{
			result = i;
			furthest = dist;
		}
	
	}
	
	return result;

}

int Formation::getClosestPointToPrint(vector<Point2f> points, PawPrint* print)
{

	int i;
	int result = 0;
	int shortest = INT_MAX;
	int dist;
	
	for(i = 0; i < (int)points.size(); i ++)
	{
	
		dist = Util::length(print -> getCenter() - points[i]);
		if(dist < shortest)
		{
			result = i;
			shortest = dist;
		}
	
	}
	
	return result;

}
