#include "util.h"

#include <opencv/cv.h>
#include <opencv/highgui.h>

#include <cmath>

using namespace cv;

// returns the minimum of two numbers
int Util::min(int v1, int v2)
{
	if(v1 > v2)
		return v2;
	return v1;
}

// returns the maximum of two numbers
int Util::max(int v1, int v2)
{
	if(v1 > v2)
		return v1;
	return v2;
}

// returns the average of 2 points in space
Point2f Util::average(Point2f a, Point2f b)
{

	Point2f ab = a + b;
	ab.x /= 2.0;
	ab.y /= 2.0;
	
	return ab;

}

// returns the average of 3 points in space
Point2f Util::average(Point2f a, Point2f b, Point2f c)
{

	Point2f ab = average(a, b);
	Point2f ac = average(a, c);
	Point2f bc = average(b, c);
	Point2f abc;
	
	abc = ab + ac + bc;
	abc.x /= 3.0;
	abc.y /= 3.0;
	
	return abc;

}

// returns the average of a vector of points in space
Point2f Util::average(vector<Point2f> points)
{

	Point2f result(0.0, 0.0);
	vector<Point2f>::iterator i;
	
	for(i = points.begin(); i != points.end(); i ++)
	{
		result = result + *i;
	}
	
	result.x /= (float)points.size();
	result.y /= (float)points.size();
	
	return result;

}

// returns the length of a given vector
float Util::length(Point2f v)
{
	return sqrt((v.x * v.x) + (v.y * v.y));
}

// returns the distance between two points in space
float Util::dist(Point2f a, Point2f b)
{
	return length(a - b);
}

// returns the angle of a given vector, from -180 to 180
float Util::angle(Point2f v)
{
	return 180.0 - radToDeg(atan2(v.x, v.y));
}

// normalizes the given vector
Point2f Util::normalize(Point2f v)
{

	const float LENGTH = length(v);
	Point2f result = v;

    result.x /= LENGTH;
    result.y /= LENGTH;
    
    return result;

}

Point2f Util::vecAtAngle(float angle)
{
	return Point2f(sinf(degToRad(angle)), cosf(degToRad(angle)));
}

// converts radians to degrees
float Util::radToDeg(float rads)
{
	return rads * 180.0f / (atanf(1.0f) * 4.0f);
}

// converts degrees to radius
float Util::degToRad(float degrees)
{
    return degrees * ((atanf(1.0f) * 4.0f) / 180.0f);
}

// wraps the given angle between 0 and 360
float Util::wrapvalue(float val)
{

    float result = val;
    
    if(val >= 360.0)
        result = val - 360.0;
    else if(val < 0.0)
        result = 360.0 + val;
    
    return result;

}

Vec3b Util::matToVec3b(Mat m)
{
	return Vec3b(m.at<float>(0, 0), m.at<float>(0, 1), m.at<float>(0, 2));
}

Mat Util::vec3bToMat(Vec3b v)
{
	return (Mat_<float>(3,1) << v[0], v[1], v[2]);
}

Point2f Util::keyPointToPoint(KeyPoint keyPoint)
{
	return keyPoint.pt;
}

int Util::getFurthestFromAverage(vector<Point2f> points)
{

	Point2f average = Util::average(points);
	
	int i;
	int furthest = 0;
	int furthestDist = 0;
	int dist;
	
	for(i = 0; i < (int)points.size(); i ++)
	{
		
		dist = length(points[i] - average);
		if(dist > furthestDist)
		{
			furthest = i;
			furthestDist = dist;
		}
		
	}

	return furthest;

}

float Util::areaToRadius(float a)
{
	return sqrt(a / 3.14159);
}

void Util::sortKeyPointsDownBySize(vector<KeyPoint>& keypoints)
{

	int i, j;
	KeyPoint curr;

	for(i = 1; i < (int)keypoints.size(); i ++)
	{

		curr = keypoints[i];

		for(j = i; j > 0 && keypoints[j - 1].size < curr.size; j --)
		{
			keypoints[j] = keypoints[j - 1];
		}

		keypoints[j] = curr;

	}

}
